@@ -1,5 +1,6 @@
 /*   GTimeTracker - a time tracker
  *   Copyright (C) 1997,98 Eckehard Berns
+ *   Copyright (C) 2001 Linas Vepstas <linas@linas.org>
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,19 +23,25 @@
 #include <gconf/gconf.h>
 #include <glade/glade.h>
 #include <gnome.h>
+#include <guile/gh.h>
 #include <libgnomeui/gnome-window-icon.h>
 #include <signal.h>
 #include <string.h>
 #include <unistd.h>
 
+#include "app.h"
 #include "ctree.h"
 #include "cur-proj.h"
 #include "dialog.h"
 #include "err-throw.h"
 #include "file-io.h"
 #include "gtt.h"
+#include "log.h"
+#include "menus.h"
 #include "menucmd.h"
+#include "prefs.h"
 #include "shorts.h"		/* SMH 2000-03-22: connect_short_cuts() */
+#include "timer.h"
 #include "xml-gtt.h"
 
 
@@ -47,10 +54,10 @@ char *first_proj_title = NULL;  /* command line over-ride */
 const char *
 gtt_gettext(const char *s)
 {
-        g_return_val_if_fail(s != NULL, NULL);
-        if (0 == strncmp(s, "[GTT]", 5))
-                return &s[5];
-        return s;
+	g_return_val_if_fail(s != NULL, NULL);
+	if (0 == strncmp(s, "[GTT]", 5))
+		return &s[5];
+	return s;
 }
 
 
@@ -101,20 +108,20 @@ static void lock_gtt(void)
 		fclose(f);
 	}
 		
-	if (warn) {
+	if (warn) 
+	{
 		GtkWidget *warning;
-#ifdef DEBUG
-                g_warning("GTT PID file exists");
-#else /* not DEBUG */
-		warning = gnome_message_box_new(_("There seems to be another GTimeTracker running.\n"
-						  "Press OK to start GTimeTracker anyway, or press Cancel to quit."),
-						GNOME_MESSAGE_BOX_WARNING,
-						GNOME_STOCK_BUTTON_OK,
-						GNOME_STOCK_BUTTON_CANCEL,
-						NULL);
+		warning = gnome_message_box_new(
+			_("There seems to be another GTimeTracker running.\n"
+			  "Press OK to start GTimeTracker anyway, or press Cancel to quit."),
+			GNOME_MESSAGE_BOX_WARNING,
+			GNOME_STOCK_BUTTON_OK,
+			GNOME_STOCK_BUTTON_CANCEL,
+			NULL);
 		if(gnome_dialog_run_and_close(GNOME_DIALOG(warning))!=0)
+		{
 			exit(0);
-#endif /* not DEBUG */
+		}
 	}
 	if (NULL == (f = fopen(fname, "wt"))) {
 		g_warning(_("Cannot create pid-file!"));
@@ -132,80 +139,212 @@ unlock_gtt(void)
 	unlink(build_lock_fname());
 }
 
-
-
-static void 
-init_list_2(GtkWidget *w, gint butnum)
+static void
+post_read_data(void)
 {
-	if (butnum == 1)
-		gtk_main_quit();
-	else
-                setup_ctree();
+	gtt_post_data_config();
+
+	err_init();
+	ctree_setup(global_ptw);
+	init_timer();
+
+	/* plugins need to be added to the main menus dynamically,
+	 * after the config file has been read */
+	menus_add_plugins (GNOME_APP(window));
+	log_start();
+	app_show();
 }
 
 static void 
-init_list(void)
+read_data_err_run_or_abort (GtkWidget *w, gint butnum)
 {
-	GttErrCode xml_errcode, conf_errcode;
+	if (butnum == 1)
+	{
+		gtk_main_quit();
+	}
+	else
+	{
+		post_read_data();
+	}
+}
+
+static const char *
+resolve_path (const char * pathfrag)
+{
+	const char * fullpath;
+
+	if (('~' != pathfrag[0]) &&
+	    ('/' != pathfrag[0]))
+	{
+		/* if not an absolute filepath ..*/
+		fullpath = gnome_config_get_real_path (pathfrag);
+	}
+	else
+	{
+		/* I suppose we should look up $HOME if ~ */
+		fullpath = pathfrag;
+	}
+
+	return fullpath;
+}
+
+
+static void 
+read_data(void)
+{
+	GttErrCode xml_errcode;
 	const char * xml_filepath;
 
-	/* Read the data file first, and the config later.
-	 * The config cile contains things like the 'current project',
-	 * which is undefined until the proejcts have been read in.
-	 */
-	xml_filepath = gnome_config_get_real_path (XML_DATA_FILENAME);
-        gtt_err_set_code (GTT_NO_ERR);
-        gtt_xml_read_file (xml_filepath);
+	xml_filepath = resolve_path (config_data_url);
 
+	/* Try ... */
+	gtt_err_set_code (GTT_NO_ERR);
+	gtt_xml_read_file (xml_filepath);
+
+	/* Catch ... */
 	xml_errcode = gtt_err_get_code();
 
-        gtt_err_set_code (GTT_NO_ERR);
-	gtt_load_config (NULL);
-	conf_errcode = gtt_err_get_code();
-
 	/* If the xml file read bombed because the file doesn't exist,
 	 * and yet the project list isn't null, that's because we read
 	 * and old-format config file that had the proejcts in it.
-	 * This is not an arror. This is OK.
+	 * This is not an error. This is OK.
 	 */
 	if (!((GTT_NO_ERR == xml_errcode) ||
 	      ((GTT_CANT_OPEN_FILE == xml_errcode) &&
-	        gtt_get_project_list())
+		gtt_get_project_list())
 	    ))
 	{
-		g_warning ("xml file read bombed, errcode = %d\n", xml_errcode);
-	}
+		const char *errmsg, *qmsg;
+		errmsg = gtt_err_to_string (xml_errcode, xml_filepath);
+		qmsg = g_strconcat (errmsg, 
+			_("Do you want to continue?"),
+			NULL);
 
+		msgbox_ok_cancel(_("Error"),
+			 qmsg,
+			 GNOME_STOCK_BUTTON_YES, 
+			 GNOME_STOCK_BUTTON_NO,
+			 GTK_SIGNAL_FUNC(read_data_err_run_or_abort));
+		g_free ((gchar *) qmsg); 
+		g_free ((gchar *) errmsg);
+	}
+	else
+	{
+		post_read_data ();
+	}
+}
+
+static void
+post_read_config(void)
+{
+	read_data();
+}
+
+static void 
+read_config_err_run_or_abort (GtkWidget *w, gint butnum)
+{
+	if (butnum == 1)
+	{
+		gtk_main_quit();
+	}
+	else
+	{
+		post_read_config();
+	}
+}
+
+static void 
+read_config(void)
+{
+	GttErrCode conf_errcode;
+
+	/* Try ... */
+	gtt_err_set_code (GTT_NO_ERR);
+	gtt_load_config (NULL);
+
+	/* Catch ... */
+	conf_errcode = gtt_err_get_code();
 	if (GTT_NO_ERR != conf_errcode) 
 	{
-                if (errno == ENOENT) {
-                        errno = 0;
-                        setup_ctree();
-                        return;
-                }
-		msgbox_ok_cancel(_("Error"),
-				 _("An error occured while reading the "
-                                   "configuration file.\n"
-				   "Shall I setup a new configuration?"),
-				 GNOME_STOCK_BUTTON_YES, 
-				 GNOME_STOCK_BUTTON_NO,
-				 GTK_SIGNAL_FUNC(init_list_2));
-	} else {
-                setup_ctree();
-	}
+		const char *fp, *errmsg, *qmsg;
+		if (errno == ENOENT) 
+		{
+			errno = 0;
+			return;
+		}
 
+		fp = gtt_get_config_filepath();
+		errmsg = gtt_err_to_string (conf_errcode, fp);
+		qmsg = g_strconcat (errmsg, 
+			_("Shall I setup a new configuration?"),
+			NULL);
+
+		msgbox_ok_cancel(_("Error"),
+			 qmsg,
+			 GNOME_STOCK_BUTTON_YES, 
+			 GNOME_STOCK_BUTTON_NO,
+			 GTK_SIGNAL_FUNC(read_config_err_run_or_abort));
+		g_free ((gchar *) qmsg); 
+		g_free ((gchar *) errmsg);
+	}
+	else 
+	{
+		post_read_config();
+	}
 }
 
 
-void
+
+static void 
+beta_run_or_abort(GtkWidget *w, gint butnum)
+{
+	if (butnum == 1)
+	{
+		gtk_main_quit();
+	}
+	else
+	{
+		read_config();
+	}
+}
+
+/* save_all() is a bit sloppy, in that if we get two errors in a row,
+ * we'll miss the second one ... but what the hey, who cares.
+ */
+
+const char *
 save_all (void)
 {
+	GttErrCode errcode;
+	const char *errmsg = NULL;
 	const char * xml_filepath;
-	xml_filepath = gnome_config_get_real_path (XML_DATA_FILENAME);
 
+	xml_filepath = resolve_path (config_data_url);
+	/* Try ... */
+	gtt_err_set_code (GTT_NO_ERR);
+	gtt_xml_write_file (xml_filepath);
+
+	/* Catch */
+	errcode = gtt_err_get_code();
+	if (GTT_NO_ERR != errcode)
+	{
+		errmsg = gtt_err_to_string (errcode, xml_filepath);
+	}
+
+	/* Try ... */
+	gtt_err_set_code (GTT_NO_ERR);
 	gtt_save_config (NULL);
 
-	gtt_xml_write_file (xml_filepath);
+	/* Catch */
+	errcode = gtt_err_get_code();
+	if (GTT_NO_ERR != errcode)
+	{
+		const char *fp;
+		fp = gtt_get_config_filepath();
+		errmsg = gtt_err_to_string (errcode, fp);
+	}
+
+	return errmsg;
 }
 
 
@@ -227,6 +366,7 @@ save_state(GnomeClient *client, gint phase, GnomeRestartStyle save_style,
 	   gint shutdown, GnomeInteractStyle interact_styyle, gint fast,
 	   gpointer data)
 {
+	const char *errmsg;
 	char *sess_id;
 	char *argv[5];
 	int argc;
@@ -253,11 +393,11 @@ save_state(GnomeClient *client, gint phase, GnomeRestartStyle save_style,
 	gnome_client_set_restart_command(client, argc, argv);
 	g_free(argv[2]);
 
-	/* save both te user preferences/config and the project lists */
-	gtt_err_set_code (GTT_NO_ERR);
-	save_all();
+	/* save both the user preferences/config and the project lists */
+	errmsg = save_all();
 	rc = 0;
-	if (GTT_NO_ERR == gtt_err_get_code()) rc = 1;
+	if (NULL == errmsg) rc = 1;
+	g_free ((gchar *) errmsg);
 
 	return rc;
 }
@@ -282,20 +422,14 @@ got_signal (int sig)
 	kill (getpid (), sig);
 }
 
-static void 
-beta_run_or_abort(GtkWidget *w, gint butnum)
-{
-	if (butnum == 1)
-	{
-		gtk_main_quit();
-	}
-	else
-	{
-		init_list();
-		log_start();
-	}
-}
 
+static void 
+guile_inner_main(int argc, char **argv)
+{
+	gtk_main();
+
+	unlock_gtt();
+}
 
 
 int 
@@ -329,25 +463,27 @@ main(int argc, char *argv[])
 			   GTK_SIGNAL_FUNC(session_die), NULL);
 #endif /* USE_SM */
 
+	glade_gnome_init();
+
+	/* gconf init is needed by gtkhtml */
+	gconf_init (argc, argv, NULL);
+
 	signal (SIGCHLD, SIG_IGN);
 	signal (SIGINT, got_signal);
 	signal (SIGTERM, got_signal);
 	lock_gtt();
 	app_new(argc, argv, geometry_string);
 
-	glade_gnome_init();
 	gtk_signal_connect(GTK_OBJECT(window), "delete_event",
 			   GTK_SIGNAL_FUNC(quit_app), NULL);
 
-	/* gconf init is needed by gtkhtml */
-	gconf_init (argc, argv, NULL);
-
 	/*
 	 * Added by SMH 2000-03-22:
 	 * Connect short-cut keys. 
 	 */
 	connect_short_cuts();
 
+#if 0
 	msgbox_ok_cancel(_("Warning"),
 		"WARNING !!! Achtung !!! Avertisment !!!\n"
 		"\n"
@@ -361,10 +497,13 @@ main(int argc, char *argv[])
 		"cvs checkout -D \"Aug 27 2001\" gnome-utils/gtt\n",
 	     "Continue", "Exit", 
 		GTK_SIGNAL_FUNC(beta_run_or_abort));
+#else
+		read_config();
+#endif
 
-	gtk_main();
 
-	unlock_gtt();
-	return 0;
+	gh_enter(argc, argv, guile_inner_main);
+	return 0; /* not reached !? */
 }
 
+
