@@ -20,21 +20,29 @@
 #include "config.h"
 
 #include <errno.h>
+#include <gnome.h>    /* only needed for definition of _() */
+#include <libintl.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 
 #include "err-throw.h"
+#include "log.h"
 #include "proj.h"
 #include "proj_p.h"
 
-static void project_list_time_reset (void);
+typedef struct notif_s 
+{
+	GttProjectChanged func;
+	gpointer user_data;
+} Notifier;
 
-// hack alert -- should be static
+// hack alert -- should be static/private to this file
 GList * plist = NULL;
 
 static void proj_refresh_time (GttProject *proj);
 static void proj_modified (GttProject *proj);
+static int task_suspend (GttTask *tsk);
 
 /* ============================================================= */
 
@@ -50,23 +58,40 @@ gtt_project_new(void)
 	proj->desc = g_strdup ("");
 	proj->notes = g_strdup ("");
 	proj->custid = NULL;
+
 	proj->min_interval = 3;
 	proj->auto_merge_interval = 60;
 	proj->auto_merge_gap = 60;
-	proj->secs_ever = 0;
-	proj->secs_day = 0;
-	proj->billrate = 1.0;
-	proj->overtime_rate = 1.5;
-	proj->overover_rate = 2.0;
-	proj->flat_fee = 1.0;
+
+	proj->billrate = 10.0;
+	proj->overtime_rate = 15.0;
+	proj->overover_rate = 20.0;
+	proj->flat_fee = 1000.0;
+
+	proj->estimated_start = -1;
+	proj->estimated_end = -1;
+	proj->due_date = -1;
+	proj->sizing = 0;
+	proj->percent_complete = 0;
+	proj->urgency = GTT_UNDEFINED;
+	proj->importance = GTT_UNDEFINED;
+	proj->status = GTT_NOT_STARTED;
+
 	proj->task_list = NULL;
 	proj->sub_projects = NULL;
 	proj->parent = NULL;
 	proj->listeners = NULL;
+        proj->private_data = NULL;
+
 	proj->being_destroyed = FALSE;
 	proj->frozen = FALSE;
 	proj->dirty_time = FALSE;
-        proj->trow = NULL;
+
+	proj->secs_ever = 0;
+	proj->secs_day = 0;
+	proj->secs_week = 0;
+	proj->secs_month = 0;
+	proj->secs_year = 0;
 
         proj->id = next_free_id;
 	next_free_id ++;
@@ -82,7 +107,7 @@ gtt_project_new_title_desc(const char *t, const char *d)
 
 	proj = gtt_project_new();
 	if (t) { g_free(proj->title); proj->title = g_strdup (t); }
-        if (d) { g_free(proj->desc); proj->desc = g_strdup (d); }
+	if (d) { g_free(proj->desc); proj->desc = g_strdup (d); }
 	return proj;
 }
 
@@ -96,11 +121,6 @@ gtt_project_dup(GttProject *proj)
 
 	if (!proj) return NULL;
 	p = gtt_project_new();
-	p->min_interval = proj->min_interval;
-	p->auto_merge_interval = proj->auto_merge_interval;
-	p->auto_merge_gap = proj->auto_merge_gap;
-	p->secs_ever = proj->secs_ever;
-	p->secs_day = proj->secs_day;
 
 	g_free(p->title);
 	g_free(p->desc);
@@ -111,11 +131,25 @@ gtt_project_dup(GttProject *proj)
 	p->notes = g_strdup(proj->notes);
 	if (proj->custid) p->custid = g_strdup(proj->custid);
 
+	p->min_interval = proj->min_interval;
+	p->auto_merge_interval = proj->auto_merge_interval;
+	p->auto_merge_gap = proj->auto_merge_gap;
+
 	p->billrate = proj->billrate;
 	p->overtime_rate = proj->overtime_rate;
 	p->overover_rate = proj->overover_rate;
 	p->flat_fee = proj->flat_fee;
 
+	p->estimated_start = proj->estimated_start;
+	p->estimated_end = proj->estimated_end;
+	p->due_date = proj->due_date;
+	p->sizing = proj->sizing;
+	p->percent_complete = proj->percent_complete;
+	p->urgency = proj->urgency;
+	p->importance = proj->importance;
+	p->status = proj->status;
+	
+
 	/* Don't copy the tasks.  Do copy the sub-projects */
 	for (node=proj->sub_projects; node; node=node->next)
 	{
@@ -187,6 +221,19 @@ gtt_project_destroy(GttProject *proj)
 		}
 	}
 
+	/* remove notifiers as well */
+	{
+		Notifier * ntf;
+		GList *node;
+	
+		for (node = proj->listeners; node; node=node->next)
+		{
+			ntf = node->data;
+			g_free (ntf);
+		}
+		if (proj->listeners) g_list_free (proj->listeners);
+	}
+	proj->private_data = NULL;
 	g_free(proj);
 }
 
@@ -350,7 +397,7 @@ gtt_project_set_min_interval (GttProject *proj, int r)
 int
 gtt_project_get_min_interval (GttProject *proj)
 {
-	if (!proj) return 0.0;
+	if (!proj) return 0;
 	return proj->min_interval;
 }
 
@@ -365,7 +412,7 @@ gtt_project_set_auto_merge_interval (GttProject *proj, int r)
 int
 gtt_project_get_auto_merge_interval (GttProject *proj)
 {
-	if (!proj) return 0.0;
+	if (!proj) return 0;
 	return proj->auto_merge_interval;
 }
 
@@ -380,10 +427,146 @@ gtt_project_set_auto_merge_gap (GttProject *proj, int r)
 int
 gtt_project_get_auto_merge_gap (GttProject *proj)
 {
-	if (!proj) return 0.0;
+	if (!proj) return 0;
 	return proj->auto_merge_gap;
 }
 
+void
+gtt_project_set_private_data (GttProject *proj, gpointer data)
+{
+	if (!proj) return;
+	proj->private_data = data; 
+}
+
+gpointer
+gtt_project_get_private_data (GttProject *proj)
+{
+	if (!proj) return NULL;
+	return proj->private_data;
+}
+
+void
+gtt_project_set_estimated_start (GttProject *proj, time_t r)
+{
+	if (!proj) return;
+	proj->estimated_start = r;
+	proj_modified (proj);
+}
+
+time_t
+gtt_project_get_estimated_start (GttProject *proj)
+{
+	if (!proj) return 0;
+	return proj->estimated_start;
+}
+
+void
+gtt_project_set_estimated_end (GttProject *proj, time_t r)
+{
+	if (!proj) return;
+	proj->estimated_end = r;
+	proj_modified (proj);
+}
+
+time_t
+gtt_project_get_estimated_end (GttProject *proj)
+{
+	if (!proj) return 0;
+	return proj->estimated_end;
+}
+
+void
+gtt_project_set_due_date (GttProject *proj, time_t r)
+{
+	if (!proj) return;
+	proj->due_date = r;
+	proj_modified (proj);
+}
+
+time_t
+gtt_project_get_due_date (GttProject *proj)
+{
+	if (!proj) return 0;
+	return proj->due_date;
+}
+
+void
+gtt_project_set_sizing (GttProject *proj, int r)
+{
+	if (!proj) return;
+	proj->sizing = r;
+	proj_modified (proj);
+}
+
+int
+gtt_project_get_sizing (GttProject *proj)
+{
+	if (!proj) return 0;
+	return proj->sizing;
+}
+
+void
+gtt_project_set_percent_complete (GttProject *proj, int r)
+{
+	if (!proj) return;
+	if (0 > r) r = 0;
+	if (100 < r) r = 100;
+	proj->percent_complete = r;
+	proj_modified (proj);
+}
+
+int
+gtt_project_get_percent_complete (GttProject *proj)
+{
+	if (!proj) return 0;
+	return proj->percent_complete;
+}
+
+void
+gtt_project_set_urgency (GttProject *proj, GttRank r)
+{
+	if (!proj) return;
+	proj->urgency = r;
+	proj_modified (proj);
+}
+
+GttRank
+gtt_project_get_urgency (GttProject *proj)
+{
+	if (!proj) return 0;
+	return proj->urgency;
+}
+
+void
+gtt_project_set_importance (GttProject *proj, GttRank r)
+{
+	if (!proj) return;
+	proj->importance = r;
+	proj_modified (proj);
+}
+
+GttRank
+gtt_project_get_importance (GttProject *proj)
+{
+	if (!proj) return 0;
+	return proj->importance;
+}
+
+void
+gtt_project_set_status (GttProject *proj, GttProjectStatus r)
+{
+	if (!proj) return;
+	proj->status = r;
+	proj_modified (proj);
+}
+
+GttProjectStatus
+gtt_project_get_status (GttProject *proj)
+{
+	if (!proj) return 0;
+	return proj->status;
+}
+
 /* =========================================================== */
 
 void
@@ -435,6 +618,69 @@ get_midnight (time_t last)
 	return midnight;
 }
 
+static time_t
+get_sunday (time_t last)
+{
+	struct tm lt;
+	time_t sunday;
+
+	if (0 >= last)
+	{
+		last = time(0);
+	}
+
+	memcpy (&lt, localtime (&last), sizeof (struct tm));
+	lt.tm_sec = 0;
+	lt.tm_min = 0;
+	lt.tm_hour = 0;
+	lt.tm_mday -= lt.tm_wday;
+	sunday = mktime (&lt);
+
+	return sunday;
+}
+
+static time_t
+get_month (time_t last)
+{
+	struct tm lt;
+	time_t first;
+
+	if (0 >= last)
+	{
+		last = time(0);
+	}
+
+	memcpy (&lt, localtime (&last), sizeof (struct tm));
+	lt.tm_sec = 0;
+	lt.tm_min = 0;
+	lt.tm_hour = 0;
+	lt.tm_mday = 1;
+	first = mktime (&lt);
+
+	return first;
+}
+
+static time_t
+get_newyear (time_t last)
+{
+	struct tm lt;
+	time_t newyear;
+
+	if (0 >= last)
+	{
+		last = time(0);
+	}
+
+	memcpy (&lt, localtime (&last), sizeof (struct tm));
+	lt.tm_sec = 0;
+	lt.tm_min = 0;
+	lt.tm_hour = 0;
+	lt.tm_mday -= lt.tm_yday;
+	newyear = mktime (&lt);
+
+	return newyear;
+}
+
 void 
 gtt_project_compat_set_secs (GttProject *proj, int sever, int sday, time_t last)
 {
@@ -463,12 +709,12 @@ gtt_project_compat_set_secs (GttProject *proj, int sever, int sday, time_t last)
 		gtt_task_add_interval (tsk, ivl);
 	}
 
-	gtt_project_add_task (proj, tsk);
+	gtt_project_append_task (proj, tsk);
 
 	/* All new data will get its own task */
 	tsk = gtt_task_new ();
 	gtt_task_set_memo (tsk, _("New Task"));
-	gtt_project_add_task (proj, tsk);
+	gtt_project_append_task (proj, tsk);
 
 	proj_refresh_time (proj);
 }
@@ -608,7 +854,7 @@ gtt_project_insert_after(GttProject *p, GttProject *after_me)
 
 
 void 
-gtt_project_add_task (GttProject *proj, GttTask *task)
+gtt_project_append_task (GttProject *proj, GttTask *task)
 {
 	if (!proj || !task) return;
 
@@ -625,6 +871,34 @@ gtt_project_add_task (GttProject *proj, GttTask *task)
 	proj_refresh_time(proj);
 }
 
+void 
+gtt_project_prepend_task (GttProject *proj, GttTask *task)
+{
+	int is_running = FALSE;
+	if (!proj || !task) return;
+
+	/* if task has a different parent, then reparent */
+	if (task->parent)
+	{
+		task->parent->task_list =
+			g_list_remove (task->parent->task_list, task);
+		proj_refresh_time(task->parent);
+	}
+
+	/* avoid misplaced running intervals, stop the task */
+	if (proj->task_list)
+	{
+		GttTask *leadtask = proj->task_list->data;
+		is_running = task_suspend (leadtask);
+	}
+
+	proj->task_list = g_list_prepend (proj->task_list, task);
+	task->parent = proj;
+
+	if (is_running) gtt_project_timer_start (proj);
+	proj_refresh_time(proj);
+}
+
 
 GList * 
 gtt_project_get_children (GttProject *proj)
@@ -651,32 +925,68 @@ gtt_project_get_tasks (GttProject *proj)
 /* =========================================================== */
 /* get totals */
 
-static int
-project_list_total_secs_day (GList *prjs)
-{
-	GList *node;
-	int total = 0;
-	if (!prjs) return 0;
-
-	for (node=prjs; node; node=node->next)
-	{
-		GttProject *proj = node->data;
-		total += proj->secs_day;
-		if (proj->sub_projects)
-		{
-			total += project_list_total_secs_day (proj->sub_projects);
-		}
-	}
-	return total;
-}
 
 int
-gtt_project_list_total_secs_day (void)
+gtt_project_foreach (GttProject *prj, GttProjectCB cb, gpointer data)
 {
-	return project_list_total_secs_day(plist);
+	int rc;
+	GList *node;
+	if (!prj || !cb) return 0;
+
+	rc = cb (prj, data);
+	if (0 == rc) return 0;
+
+	for (node=prj->sub_projects; node; node=node->next)
+	{
+		GttProject *subprj = node->data;
+		if (!subprj) continue;  /* this can't really happen */
+		rc = gtt_project_foreach (subprj, cb, data);
+		if (0 == rc) return 0;
+	}
+	return rc;
 }
 
+static int
+prj_total_secs_day (GttProject *prj, gpointer data)
+{
+	*((int *) data) += prj->secs_day;
+	return 1;
+}
 
+static int
+prj_total_secs_week (GttProject *prj, gpointer data)
+{
+	*((int *) data) += prj->secs_week;
+	return 1;
+}
+
+static int
+prj_total_secs_month (GttProject *prj, gpointer data)
+{
+	*((int *) data) += prj->secs_month;
+	return 1;
+}
+
+static int
+prj_total_secs_year (GttProject *prj, gpointer data)
+{
+	*((int *) data) += prj->secs_year;
+	return 1;
+}
+
+static int
+prj_total_secs_ever (GttProject *prj, gpointer data)
+{
+	*((int *) data) += prj->secs_ever;
+	return 1;
+}
+
+static int
+prj_total_secs_current (GttProject *prj, gpointer data)
+{
+	*((int *) data) += gtt_project_get_secs_current (prj);
+	return 1;
+}
 
 /* this routine adds up total day-secs for this project, and its sub-projects */
 int
@@ -684,12 +994,52 @@ gtt_project_total_secs_day (GttProject *proj)
 {
 	int total = 0;
 	if (!proj) return 0;
+	gtt_project_foreach (proj, prj_total_secs_day, &total);
+	return total;
+}
 
-	total = proj->secs_day;
-	if (proj->sub_projects)
-	{
-		total += project_list_total_secs_day (proj->sub_projects);
-	}
+int
+gtt_project_total_secs_week (GttProject *proj)
+{
+	int total = 0;
+	if (!proj) return 0;
+	gtt_project_foreach (proj, prj_total_secs_week, &total);
+	return total;
+}
+
+int
+gtt_project_total_secs_month (GttProject *proj)
+{
+	int total = 0;
+	if (!proj) return 0;
+	gtt_project_foreach (proj, prj_total_secs_month, &total);
+	return total;
+}
+
+int
+gtt_project_total_secs_year (GttProject *proj)
+{
+	int total = 0;
+	if (!proj) return 0;
+	gtt_project_foreach (proj, prj_total_secs_year, &total);
+	return total;
+}
+
+int
+gtt_project_total_secs_ever (GttProject *proj)
+{
+	int total = 0;
+	if (!proj) return 0;
+	gtt_project_foreach (proj, prj_total_secs_ever, &total);
+	return total;
+}
+
+int
+gtt_project_total_secs_current (GttProject *proj)
+{
+	int total = 0;
+	if (!proj) return 0;
+	gtt_project_foreach (proj, prj_total_secs_current, &total);
 	return total;
 }
 
@@ -700,46 +1050,25 @@ gtt_project_get_secs_day (GttProject *proj)
 	return proj->secs_day;
 }
 
-
-static int
-project_list_total_secs_ever (GList *prjs)
-{
-	GList *node;
-	int total = 0;
-	if (!prjs) return 0;
-
-	for (node=prjs; node; node=node->next)
-	{
-		GttProject *proj = node->data;
-		total += proj->secs_ever;
-		if (proj->sub_projects)
-		{
-			total += project_list_total_secs_ever (proj->sub_projects);
-		}
-	}
-	return total;
-}
-
 int
-gtt_project_list_total_secs_ever (void)
+gtt_project_get_secs_week (GttProject *proj)
 {
-	return project_list_total_secs_ever(plist);
-}
-
-
-/* this routine adds up total secs for this project, and its sub-projects */
-int
-gtt_project_total_secs_ever (GttProject *proj)
-{
-	int total = 0;
 	if (!proj) return 0;
+	return proj->secs_week;
+}
 
-	total = proj->secs_ever;
-	if (proj->sub_projects)
-	{
-		total += project_list_total_secs_ever (proj->sub_projects);
-	}
-	return total;
+int
+gtt_project_get_secs_month (GttProject *proj)
+{
+	if (!proj) return 0;
+	return proj->secs_month;
+}
+
+int
+gtt_project_get_secs_year (GttProject *proj)
+{
+	if (!proj) return 0;
+	return proj->secs_year;
 }
 
 int
@@ -749,6 +1078,33 @@ gtt_project_get_secs_ever (GttProject *proj)
 	return proj->secs_ever;
 }
 
+int
+gtt_project_get_secs_current (GttProject *proj)
+{
+	GttTask *tsk;
+	if (!proj) return 0;
+	if (!proj->task_list) return 0;
+	tsk = proj->task_list->data;
+
+	return gtt_task_get_secs_ever(tsk);
+}
+
+/* =========================================================== */
+
+int
+gtt_project_list_total_secs_day (void)
+{
+	GList *node;
+	int total = 0;
+	if (!plist) return 0;
+
+	for (node=plist; node; node=node->next)
+	{
+		GttProject *proj = node->data;
+		total += gtt_project_total_secs_day (proj);
+	}
+	return total;
+}
 
 /* =========================================================== */
 /* Recomputed cached data.  Scrub it while we're at it. */
@@ -758,6 +1114,7 @@ gtt_project_get_secs_ever (GttProject *proj)
  *     than proj->auto_merge_gap
  *     Merge all intervals that are shorter than 
  *     proj->auto_merge_interval into the nearest interval.
+ *     (but only do this if the nearest is in the same day).
  */
 
 static int
@@ -787,6 +1144,7 @@ scrub_intervals (GttTask *tsk)
 			    ((ivl->stop - ivl->start) <= mini))
 			{
 				tsk->interval_list = g_list_remove (tsk->interval_list, ivl);
+				g_free (ivl);
 				not_done = TRUE;
 				changed = TRUE;
 				break;
@@ -833,6 +1191,7 @@ scrub_intervals (GttTask *tsk)
 			GttInterval *ivl = node->data;
 			int gap_up=1000000000;
 			int gap_down=1000000000;
+			int do_merge = FALSE;
 			int len;
 
 			if (ivl->running) continue;
@@ -841,14 +1200,26 @@ scrub_intervals (GttTask *tsk)
 			if (node->next)
 			{
 				GttInterval *nivl = node->next->data;
-				gap_down = ivl->start - nivl->stop;
+
+				/* merge only if the intervals are in the same day */
+				if (get_midnight (ivl->start) == get_midnight (nivl->stop))
+				{
+					gap_down = ivl->start - nivl->stop;
+					do_merge = TRUE;
+				}
 			} 
 			if (node->prev)
 			{
 				GttInterval *nivl = node->prev->data;
-				gap_up = nivl->start - ivl->stop;
+
+				/* merge only if the intervals are in the same day */
+				if (get_midnight (nivl->start) == get_midnight (ivl->stop))
+				{
+					gap_up = nivl->start - ivl->stop;
+					do_merge = TRUE;
+				}
 			}
-			if (!node->next && !node->prev) continue;
+			if (!do_merge) continue;
 			if (gap_up < gap_down)
 			{
 				gtt_interval_merge_up (ivl);
@@ -867,12 +1238,15 @@ scrub_intervals (GttTask *tsk)
 	return changed;
 }
 
-void
-gtt_project_compute_secs (GttProject *proj)
+static void
+project_compute_secs (GttProject *proj)
 {
 	int total_ever = 0;
 	int total_day = 0;
-	time_t midnight;
+	int total_week = 0;
+	int total_month = 0;
+	int total_year = 0;
+	time_t midnight, sunday, month, newyear;
 	GList *tsk_node, *ivl_node, *prj_node;
 
 	if (!proj) return;
@@ -881,10 +1255,14 @@ gtt_project_compute_secs (GttProject *proj)
 	for (prj_node= proj->sub_projects; prj_node; prj_node=prj_node->next)
 	{
 		GttProject * prj = prj_node->data;
-		gtt_project_compute_secs (prj);
+		project_compute_secs (prj);
 	}
 
 	midnight = get_midnight (-1);
+	sunday = get_sunday (-1);
+	month = get_month (-1);
+	newyear = get_newyear (-1);
+
 	/* total up tasks */
 	for (tsk_node= proj->task_list; tsk_node; tsk_node=tsk_node->next)
 	{
@@ -902,14 +1280,54 @@ gtt_project_compute_secs (GttProject *proj)
 			{
 				total_day += ivl->stop - midnight;
 			}
+			if (ivl->start >= sunday)
+			{
+				total_week += ivl->stop - ivl->start;
+			}
+			else if (ivl->stop > sunday)
+			{
+				total_week += ivl->stop - sunday;
+			}
+			if (ivl->start >= month)
+			{
+				total_month += ivl->stop - ivl->start;
+			}
+			else if (ivl->stop > month)
+			{
+				total_month += ivl->stop - month;
+			}
+			if (ivl->start >= newyear)
+			{
+				total_year += ivl->stop - ivl->start;
+			}
+			else if (ivl->stop > newyear)
+			{
+				total_year += ivl->stop - newyear;
+			}
 		}
 	}
 
 	proj->secs_ever = total_ever;
 	proj->secs_day = total_day;
+	proj->secs_week = total_week;
+	proj->secs_month = total_month;
+	proj->secs_year = total_year;
 	proj->dirty_time = FALSE;
 }
 
+static void
+children_modified (GttProject *prj)
+{
+	GList *node;
+	if (!prj) return;
+	for (node= prj->sub_projects; node; node=node->next)
+	{
+		GttProject * subprj = node->data;
+		children_modified (subprj);
+	}
+	proj_modified (prj);
+}
+
 void
 gtt_project_list_compute_secs (void)
 {
@@ -917,18 +1335,13 @@ gtt_project_list_compute_secs (void)
 	for (node= plist; node; node=node->next)
 	{
 		GttProject * prj = node->data;
-		gtt_project_compute_secs (prj);
+		project_compute_secs (prj);
+		children_modified (prj);
 	}
 }
 
 /* =========================================================== */
-/* even ntificatin subsystem */
-
-typedef struct notif_s 
-{
-	GttProjectChanged func;
-	gpointer user_data;
-} Notifier;
+/* even notification subsystem */
 
 void
 gtt_project_add_notifier (GttProject *prj, 
@@ -964,6 +1377,7 @@ gtt_project_remove_notifier (GttProject *prj,
 	if (node)
 	{
 		prj->listeners = g_list_remove (prj->listeners, ntf);
+		g_free (ntf);
 	}
 }
 
@@ -1021,7 +1435,7 @@ proj_refresh_time (GttProject *proj)
 	if (proj->being_destroyed) return;
 	proj->dirty_time = TRUE;
 	if (proj->frozen) return;
-	gtt_project_compute_secs (proj);
+	project_compute_secs (proj);
 
 	/* let listeners know that the times have changed */
 	for (node=proj->listeners; node; node=node->next)
@@ -1053,45 +1467,45 @@ proj_modified (GttProject *proj)
 void 
 gtt_clear_daily_counter (GttProject *proj)
 {
+	time_t midnight;
+	int not_done = TRUE;
+	GList *tsk_node, *in;
+
 	if (!proj) return;
-	proj->secs_day = 0;
-	proj_modified (proj);
-}
 
-/* =========================================================== */
-/* zero out day counts if rolled past midnight */
+	gtt_project_timer_stop (proj);
+	gtt_project_freeze (proj);
+	midnight = get_midnight (-1);
 
-static int day_last_reset = -1;
-static int year_last_reset = -1;
-
-void
-set_last_reset (time_t last)
-{
-	struct tm *t0;
-	t0 = localtime (&last);
-	day_last_reset = t0->tm_yday;
-	year_last_reset = t0->tm_year;
-}
-
-
-void
-zero_on_rollover (time_t now)
-{
-	struct tm *t1;
-
-	/* zero out day counts */
-	t1 = localtime(&now);
-	if ((year_last_reset != t1->tm_year) ||
-	    (day_last_reset != t1->tm_yday)) 
+	/* loop over tasks */
+	for (tsk_node= proj->task_list; tsk_node; tsk_node=tsk_node->next)
 	{
-		gtt_project_list_compute_secs ();
-		project_list_time_reset();
-		log_endofday();
-		year_last_reset = t1->tm_year;
-	    	day_last_reset = t1->tm_yday;
-	}
-}
+		GttTask * task = tsk_node->data;
 
+		not_done = TRUE;
+		while (not_done)
+		{
+			not_done = FALSE;
+			for (in= task->interval_list; in; in=in->next)
+			{
+				GttInterval *ivl = in->data;
+	
+				/* only nuke the ones that started after midnight.
+			 	 * The ones that started before midnight remain */
+				if (ivl->start >= midnight)
+				{
+					task->interval_list = 
+					     g_list_remove (task->interval_list, ivl);
+					g_free (ivl);
+					not_done = TRUE;
+					break;
+				}
+			}
+		}
+	}
+	gtt_project_thaw (proj);
+	gtt_project_timer_start (proj);
+}
 
 /* =========================================================== */
 
@@ -1100,25 +1514,57 @@ gtt_project_timer_start (GttProject *proj)
 {
 	GttTask *task;
 	GttInterval *ival;
+	time_t now;
 
 	if (!proj) return;
-	if (NULL == proj->task_list)
-	{
-		task = gtt_task_new();
-		proj->task_list = g_list_prepend (NULL, task);
-		gtt_task_set_memo (task, _("New Task"));
-	}
 
 	/* By definition, the current task is the one at the head 
 	 * of the list, and the current interval is  at the head 
 	 * of the task */
-	task = proj->task_list->data;
-	g_return_if_fail (task);
+	if (NULL == proj->task_list)
+	{
+		task = gtt_task_new();
+		gtt_task_set_memo (task, _("New Task"));
+	}
+	else 
+	{
+		task = proj->task_list->data;
+		g_return_if_fail (task);
+	}
+
+	now = time(0);
+
+	/* only add a new interval if there's been a bit of a gap,
+	 * otherwise, reuse the most recent running interval.  */
+	if (task->interval_list)
+	{
+		int delta;
+		ival = task->interval_list->data;
+		delta = now - ival->stop;
+
+		if (delta <= proj->auto_merge_gap)
+		{
+			
+			ival->start += delta;
+			ival->fuzz += delta;
+			ival->stop = now;
+			ival->running = TRUE;
+			return;
+		}
+	} 
+
 	ival = g_new0 (GttInterval, 1);
-	ival->start = time(0);
+	ival->start = now;
 	ival->stop = ival->start;
 	ival->running = TRUE;
 	task->interval_list = g_list_prepend (task->interval_list, ival);
+	ival->parent = task;
+
+	/* don't add the task until after we've done above */
+	if (NULL == proj->task_list)
+	{
+		gtt_project_append_task (proj, task);
+	}
 }
 
 void 
@@ -1140,26 +1586,24 @@ gtt_project_timer_update (GttProject *proj)
 	ival = task->interval_list->data;
 	g_return_if_fail (ival);
 
+	/* If timer isn't running, do nothing.  Normally,
+	 * this function should never be called when timer is stopped,
+	 * but there are a few rare cases (e.g. clear daily counter).
+	 * where it is.
+	 */
+	if (FALSE == ival->running) return;
 
 	/* compute the delta change, update cached data */
 	now = time(0);
-	zero_on_rollover (now);
-
-	/* if timer isn't running, do nothing.  Its arguabley
-	 * an error condition if this routine was called with the timer
-	 * stopped .... maybe we should printf a complaint ?
-	 */
-	if (FALSE == ival->running) 
-	{
-		g_warning ("update called while timer stopped!\n");
-		return;
-	}
 
 	prev_update = ival->stop;
 	ival->stop = now;
 	diff = now - prev_update;
 	proj->secs_ever += diff;
 	proj->secs_day += diff;
+	proj->secs_week += diff;
+	proj->secs_month += diff;
+	proj->secs_year += diff;
 }
 
 void 
@@ -1182,12 +1626,13 @@ gtt_project_timer_stop (GttProject *proj)
 
 	ival->running = FALSE;
 
-	/* do not record zero-length or very short intervals */
-	if (proj->min_interval >= (ival->stop - ival->start))
-	{
-		task->interval_list = g_list_remove (task->interval_list, ival);
-		g_free (ival);
-	}
+	/* When we stop the timer, call proj_refresh_time(),
+	 * as this will do several things: first, it will 
+	 * scrub away short intervals and/or short gaps,
+	 * and, secondly, it will force redraws of affected 
+	 * windows so that the old data doesn't show.
+	 */
+	proj_refresh_time (proj);
 }
 
 /* =========================================================== */
@@ -1199,23 +1644,72 @@ gtt_task_new (void)
 
 	task = g_new0(GttTask, 1);
 	task->parent = NULL;
-	task->memo = g_strdup ("");
+	task->memo = g_strdup (_("New Task"));
 	task->notes = g_strdup ("");
 	task->billable = GTT_BILLABLE;
 	task->billrate = GTT_REGULAR;
+	task->billstatus = GTT_BILL,
 	task->bill_unit = 900;
 	task->interval_list = NULL;
 	return task;
 }
 
+GttTask *
+gtt_task_dup (GttTask *old)
+{
+	GttTask *task;
+
+	if (!old) return gtt_task_new();
+
+	task = g_new0(GttTask, 1);
+	task->parent = NULL;
+	task->memo = g_strdup (old->memo);
+	task->notes = g_strdup (old->notes);
+
+	/* inherit the properties ... important for user */
+	task->billable = old->billable;
+	task->billrate = old->billrate;
+	task->billstatus = old->billstatus;
+	task->bill_unit = old->bill_unit;
+	task->interval_list = NULL;
+
+	return task;
+}
+
+static int
+task_suspend (GttTask *tsk)
+{
+	int is_running = 0;
+
+	/* avoid misplaced running intervals, stop the task */
+	if (tsk->interval_list)
+	{
+		GttInterval *first_ivl;
+		first_ivl = (GttInterval *) (tsk->interval_list->data);
+		is_running = first_ivl -> running;
+		if (is_running) 
+		{
+			/* don't call stop here, avoid dispatching redraw events */
+			gtt_project_timer_update (tsk->parent);
+			first_ivl->running = FALSE;
+		}
+	}
+	return is_running;
+}
+
+
 void
 gtt_task_destroy (GttTask *task)
 {
+	int is_running;
 	if (!task) return;
+
+	is_running = task_suspend (task);
 	if (task->parent)
 	{
 		task->parent->task_list = 
 			g_list_remove (task->parent->task_list, task);
+		if (is_running) gtt_project_timer_start (task->parent);
 		proj_refresh_time (task->parent);
 		task->parent = NULL;
 	}
@@ -1237,6 +1731,84 @@ gtt_task_destroy (GttTask *task)
 	}
 }
 
+void
+gtt_task_remove (GttTask *task)
+{
+	int is_running;
+	if (!task) return;
+
+	is_running = task_suspend (task);
+
+	if (task->parent)
+	{
+		task->parent->task_list = 
+			g_list_remove (task->parent->task_list, task);
+		if (is_running) gtt_project_timer_start (task->parent);
+		proj_refresh_time (task->parent);
+		task->parent = NULL;
+	}
+}
+
+
+void
+gtt_task_insert (GttTask *where, GttTask *insertee)
+{
+	GttProject *prj;
+	int idx;
+	int is_running = 0;
+
+	if (!where || !insertee) return;
+
+	prj = where->parent;
+	if (!prj) return;
+
+	gtt_task_remove (insertee);
+	is_running = task_suspend (where);
+
+	insertee->parent = prj;
+	idx = g_list_index (prj->task_list, where);
+	prj->task_list = g_list_insert (prj->task_list, insertee, idx);
+
+	if (is_running) gtt_project_timer_start (prj);
+	proj_refresh_time (prj);
+}
+
+GttTask *
+gtt_task_new_insert (GttTask *old)
+{
+	int is_running = 0;
+	GttProject *prj;
+	GttTask *task;
+	int idx;
+
+	if (!old) return NULL;
+
+	task = g_new0(GttTask, 1);
+	task->memo = g_strdup (_("New Task"));
+	task->notes = g_strdup ("");
+
+	/* inherit the properties ... important for user */
+	task->billable = old->billable;
+	task->billrate = old->billrate;
+	task->billstatus = old->billstatus;
+	task->bill_unit = old->bill_unit;
+	task->interval_list = NULL;
+
+	/* chain into place */
+	prj = old->parent;
+	task->parent = prj;
+	if (!prj) return task;
+
+	/* avoid misplaced running intervals, stop the task */
+	is_running = task_suspend (old);
+
+	idx = g_list_index (prj->task_list, old);
+	prj->task_list = g_list_insert (prj->task_list, task, idx);
+
+	if (is_running) gtt_project_timer_start (prj);
+	proj_refresh_time (prj);
+	return task;
+}
 
 void
 gtt_task_add_interval (GttTask *tsk, GttInterval *ival)
@@ -1309,7 +1881,7 @@ gtt_task_set_billable (GttTask *tsk, GttBillable b)
 GttBillable
 gtt_task_get_billable (GttTask *tsk)
 {
-	if (!tsk) return GTT_HOLD;
+	if (!tsk) return GTT_NOT_BILLABLE;
 	return tsk->billable;
 }
 
@@ -1328,6 +1900,21 @@ gtt_task_get_billrate (GttTask *tsk)
 	return tsk->billrate;
 }
 
+void
+gtt_task_set_billstatus (GttTask *tsk, GttBillStatus b)
+{
+	if (!tsk) return;
+	tsk->billstatus = b;
+	proj_modified (tsk->parent);
+}
+
+GttBillStatus
+gtt_task_get_billstatus (GttTask *tsk)
+{
+	if (!tsk) return GTT_HOLD;
+	return tsk->billstatus;
+}
+
 void
 gtt_task_set_bill_unit (GttTask *tsk, int b)
 {
@@ -1350,6 +1937,45 @@ gtt_task_get_intervals (GttTask *tsk)
 	return tsk->interval_list;
 }
 
+gboolean
+gtt_task_is_first_task (GttTask *tsk)
+{
+	if (!tsk || !tsk->parent || !tsk->parent->task_list) return TRUE;
+	
+	if ((GttTask *) tsk->parent->task_list->data == tsk) return TRUE;
+	return FALSE;
+}
+
+/* =========================================================== */
+
+void 
+gtt_task_merge_up (GttTask *tsk)
+{
+	GttTask *mtask;
+	GList * node;
+	GttProject *prj;
+	if (!tsk) return;
+
+	prj = tsk->parent;
+	if (!prj || !prj->task_list) return;
+
+	node = g_list_find (prj->task_list, tsk);
+	if (!node) return;
+
+	node = node->prev;
+	if (!node) return;
+
+	mtask = node->data;
+
+	for (node=tsk->interval_list; node; node=node->next)
+	{
+		GttInterval *ivl = node->data;
+		ivl->parent = mtask;
+	}
+	mtask->interval_list = g_list_concat (mtask->interval_list, tsk->interval_list);
+	tsk->interval_list = NULL;
+}
+
 /* =========================================================== */
 
 int
@@ -1456,6 +2082,31 @@ gtt_interval_get_running (GttInterval * ivl)
 	return (gboolean) ivl->running;
 }
 
+GttTask *
+gtt_interval_get_parent (GttInterval * ivl)
+{
+	if (!ivl) return NULL;
+	return ivl->parent;
+}
+
+gboolean
+gtt_interval_is_first_interval (GttInterval *ivl)
+{
+	if (!ivl || !ivl->parent || !ivl->parent->interval_list) return TRUE;
+	
+	if ((GttInterval *) ivl->parent->interval_list->data == ivl) return TRUE;
+	return FALSE;
+}
+
+gboolean
+gtt_interval_is_last_interval (GttInterval *ivl)
+{
+	if (!ivl || !ivl->parent || !ivl->parent->interval_list) return TRUE;
+
+	if ((GttInterval *) ((g_list_last(ivl->parent->interval_list))->data) == ivl) return TRUE;
+	return FALSE;
+}
+
 /* ============================================================= */
 
 GttInterval *
@@ -1536,24 +2187,35 @@ gtt_interval_merge_down (GttInterval *ivl)
 	return merge;
 }
 
-GttTask *
-gtt_interval_split (GttInterval *ivl)
+void
+gtt_interval_split (GttInterval *ivl, GttTask *newtask)
 {
+	int is_running = 0;
 	gint idx;
 	GttProject *prj;
 	GttTask *prnt;
-	GttTask *newtask;
 	GList *node;
+	GttInterval *first_ivl;
 
-	if (!ivl) return NULL;
+	if (!ivl || !newtask) return;
 	prnt = ivl->parent;
-	if (!prnt) return NULL;
+	if (!prnt) return;
 	prj = prnt->parent;
-	if (!prj) return NULL;
+	if (!prj) return;
 	node = g_list_find (prnt->interval_list, ivl);
-	if (!node) return NULL;
+	if (!node) return;
 
-	newtask = gtt_task_new();
+	gtt_task_remove (newtask);
+
+	/* avoid misplaced running intervals, stop the task */
+	first_ivl = (GttInterval *) (prnt->interval_list->data);
+	is_running = first_ivl -> running;
+	if (is_running) 
+	{
+		/* don't call stop here, avoid dispatching redraw events */
+		gtt_project_timer_update (prj);
+		first_ivl->running = FALSE;
+	}
 
 	/* chain the new task into proper order in the parent project */
 	idx = g_list_index (prj->task_list, prnt);
@@ -1581,8 +2243,9 @@ gtt_interval_split (GttInterval *ivl)
 		nivl->parent = newtask;
 	}
 
+	if (is_running) gtt_project_timer_start (prj);
+
 	proj_refresh_time (prnt->parent);
-	return newtask;
 }
 
 /* ============================================================= */
@@ -1614,10 +2277,20 @@ project_list_destroy(void)
 	}
 }
 
-static void
-project_list_sort(int (cmp)(const void *, const void *))
+static GList *
+project_list_sort(GList *prjs, int (cmp)(const void *, const void *))
 {
-	plist = g_list_sort (plist, cmp);
+	GList *node;
+	prjs = g_list_sort (prjs, cmp);
+	for (node = prjs; node; node=node->next)
+	{
+		GttProject *prj = node->data;
+		if (prj->sub_projects)
+		{
+			prj->sub_projects = project_list_sort (prj->sub_projects, cmp);
+		}
+	}
+	return prjs;
 }
 
 /* -------------------- */
@@ -1648,39 +2321,56 @@ gtt_project_locate_from_id (int prj_id)
 	return locate_from_id (plist, prj_id);
 }
 
-/* ============================================================= */
-/* misc GUI-related routines */
-
-static void 
-project_list_time_reset(void)
-{
-	GList *node;
-	for (node = plist; node; node=node->next)
-	{
-		GttProject *prj = node->data;
-		prj->secs_day = 0;
-		proj_modified (prj);
-	}
-}
-
 /* ==================================================================== */
 /* sort funcs */
 
 
 static int
-cmp_time(const void *aa, const void *bb)
+cmp_day(const void *aa, const void *bb)
 {
 	GttProject *a = (GttProject *)aa;
 	GttProject *b = (GttProject *)bb;
-        return (gtt_project_total_secs_day(b) - gtt_project_total_secs_day(a));
+	return (gtt_project_total_secs_day(b) - gtt_project_total_secs_day(a));
 }
 
 static int
-cmp_total_time(const void *aa, const void *bb)
+cmp_week(const void *aa, const void *bb)
 {
 	GttProject *a = (GttProject *)aa;
 	GttProject *b = (GttProject *)bb;
-        return (gtt_project_total_secs_ever(b) - gtt_project_total_secs_ever(a));
+	return (gtt_project_total_secs_week(b) - gtt_project_total_secs_week(a));
+}
+
+static int
+cmp_month(const void *aa, const void *bb)
+{
+	GttProject *a = (GttProject *)aa;
+	GttProject *b = (GttProject *)bb;
+	return (gtt_project_total_secs_month(b) - gtt_project_total_secs_month(a));
+}
+
+static int
+cmp_year(const void *aa, const void *bb)
+{
+	GttProject *a = (GttProject *)aa;
+	GttProject *b = (GttProject *)bb;
+	return (gtt_project_total_secs_year(b) - gtt_project_total_secs_year(a));
+}
+
+static int
+cmp_ever(const void *aa, const void *bb)
+{
+	GttProject *a = (GttProject *)aa;
+	GttProject *b = (GttProject *)bb;
+	return (gtt_project_total_secs_ever(b) - gtt_project_total_secs_ever(a));
+}
+
+static int
+cmp_current(const void *aa, const void *bb)
+{
+	GttProject *a = (GttProject *)aa;
+	GttProject *b = (GttProject *)bb;
+	return (gtt_project_total_secs_current(b) - gtt_project_total_secs_current(a));
 }
 
 
@@ -1689,7 +2379,7 @@ cmp_title(const void *aa, const void *bb)
 {
 	const GttProject *a = aa;
 	const GttProject *b = bb;
-        return strcmp(a->title, b->title);
+	return strcmp(a->title, b->title);
 }
 
 static int
@@ -1706,29 +2396,167 @@ cmp_desc(const void *aa, const void *bb)
 	return strcmp(a->desc, b->desc);
 }
 
+static int
+cmp_start(const void *aa, const void *bb)
+{
+	const GttProject *a = aa;
+	const GttProject *b = bb;
+	return (b->estimated_start - a->estimated_start);
+}
+
+static int
+cmp_end(const void *aa, const void *bb)
+{
+	const GttProject *a = aa;
+	const GttProject *b = bb;
+	return (b->estimated_end - a->estimated_end);
+}
+
+
+static int
+cmp_due(const void *aa, const void *bb)
+{
+	const GttProject *a = aa;
+	const GttProject *b = bb;
+	return (b->due_date - a->due_date);
+}
+
+
+static int
+cmp_sizing(const void *aa, const void *bb)
+{
+	const GttProject *a = aa;
+	const GttProject *b = bb;
+	return (b->sizing - a->sizing);
+}
+
+static int
+cmp_percent(const void *aa, const void *bb)
+{
+	const GttProject *a = aa;
+	const GttProject *b = bb;
+	return (b->percent_complete - a->percent_complete);
+}
+
+static int
+cmp_urgency(const void *aa, const void *bb)
+{
+	const GttProject *a = aa;
+	const GttProject *b = bb;
+	return (b->urgency - a->urgency);
+}
+
+static int
+cmp_importance(const void *aa, const void *bb)
+{
+	const GttProject *a = aa;
+	const GttProject *b = bb;
+	return (b->importance - a->importance);
+}
+
+static int
+cmp_status(const void *aa, const void *bb)
+{
+	const GttProject *a = aa;
+	const GttProject *b = bb;
+	return (b->status - a->status);
+}
+
 
 void
-project_list_sort_time(void)
+project_list_sort_day(void)
 {
-        project_list_sort(cmp_time);
+	plist = project_list_sort(plist, cmp_day);
 }
 
 void
-project_list_sort_total_time(void)
+project_list_sort_week(void)
 {
-	project_list_sort(cmp_total_time);
+	plist = project_list_sort(plist, cmp_week);
+}
+
+void
+project_list_sort_month(void)
+{
+	plist = project_list_sort(plist, cmp_month);
+}
+
+void
+project_list_sort_year(void)
+{
+	plist = project_list_sort(plist, cmp_year);
+}
+
+void
+project_list_sort_ever(void)
+{
+	plist = project_list_sort(plist, cmp_ever);
+}
+
+void
+project_list_sort_current(void)
+{
+	plist = project_list_sort(plist, cmp_current);
 }
 
 void
 project_list_sort_title(void)
 {
-        project_list_sort(cmp_title);
+	plist = project_list_sort(plist, cmp_title);
 }
 
 void
 project_list_sort_desc(void)
 {
-	project_list_sort(cmp_desc);
+	plist = project_list_sort(plist, cmp_desc);
+}
+
+void
+project_list_sort_start(void)
+{
+	plist = project_list_sort(plist, cmp_start);
+}
+
+void
+project_list_sort_end(void)
+{
+	plist = project_list_sort(plist, cmp_end);
+}
+
+void
+project_list_sort_due(void)
+{
+	plist = project_list_sort(plist, cmp_due);
+}
+
+void
+project_list_sort_sizing(void)
+{
+	plist = project_list_sort(plist, cmp_sizing);
+}
+
+void
+project_list_sort_percent(void)
+{
+	plist = project_list_sort(plist, cmp_percent);
+}
+
+void
+project_list_sort_urgency(void)
+{
+	plist = project_list_sort(plist, cmp_urgency);
+}
+
+void
+project_list_sort_importance(void)
+{
+	plist = project_list_sort(plist, cmp_importance);
+}
+
+void
+project_list_sort_status(void)
+{
+	plist = project_list_sort(plist, cmp_status);
 }
 
 /* =========================== END OF FILE ========================= */
